##############################################################################
##
##   $Id: Reports.pm,v 1.2 2004/04/03 14:04:53 croftj Exp $
##   Copyright (c) 2004 Joe Croft <joe@croftj.net>
##   
##   This file is part of ah2.
##
##   ah2 is free software; you can redistribute it and/or modify
##   it under the terms of the GNU General Public License as published by
##   the Free Software Foundation; either version 2 of the License, or
##   (at your option) any later version.
##
##   Foobar is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU General Public License for more details.
##
##   You should have received a copy of the GNU General Public License
##   along with Foobar; if not, write to the Free Software
##   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##############################################################################

package        Reports;
require        Exporter;
@ISA           = qw(Exporter);
@EXPORT        = qw(getDateSpan, getNextMonth, openDatabase, print_rpt, fmt_money);

my @days = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);

sub new
{
   my ($class, $args, $ppopts) = @_;
   my $self = {};

   $self->{detailed} = 0;
   $self->{done} = 0;
   $self->{monthly} = $cgi;

   my ($n, $n, $n, $day, $mon, $yr, $n, $n, $n) = localtime();
   $yr += 1900;
   $self->{beg} = "1/1/$yr";
   $self->{end} = "12/31/$yr";
   $mon++;

   while ($args->[0] ne undef) {
#      print "Have args->[0] = |$args->[0]|\n";
      $arg = shift @$args;
#      print "Have arg = |$arg|\n";
      ($cmd, $val) = split /=/,$arg;
#      print "Have cmd = |$cmd|, val = |$val|\n";

      if ( $cmd =~ /printer/i ) {
         $self->{printer} = $val;
      }
      elsif ( $cmd =~ /format/i ) {
         $self->{out_format} = $val;
      }
      elsif ( $cmd =~ /beg/i ) {
         $self->{beg} = $val;
         my ($m1, $d1, $y1) = split /\//, $self->{beg};
         $self->{day} = $d1;
         $self->{mon} = $m1;
         $self->{yr} = $y1;
      }
      elsif ( $cmd =~ /end/i ) {
         $self->{end} = $val;
      }
      elsif ( $cmd =~ /title/i ) {
         $self->{title} = $val;
      }
      elsif ( $cmd =~ /type/i ) {
         $self->{rpt_type} = $val;
         if ( $val eq "year_end" ) {
            $self->{beg} = "1/1/$yr";
            $self->{end} = "12/$days[12-1]/$yr";
            $self->{type} = "Year End";
         }
         elsif ( $val eq "mon" ) {
            my $mon = $self->{mon};
            my $yr = $self->{yr};
            $self->{beg} = "$mon/1/$yr";
            $self->{end} = "$mon/$days[$mon-1]/$yr";
            $self->{type} = "Month of $mon";
         }
         elsif ( $val eq "ytod" ) {
            $self->{beg} = "1/1/$yr";
            $self->{end} = "$mon/$day/$yr";
            $self->{type} = "Year to Date";
         }
         elsif ( $val eq "mtod" ) {
            $self->{beg} = "$mon/1/$yr";
            $self->{end} = "$mon/$day/$yr";
            $self->{type} = "Month to Date";
         }
      }
      elsif ( $cmd =~ /detailed/i ) {
         if ( $val eq "Y" ) {
            $self->{detailed} = 1;
            if ( "$type" eq "" ) {
               $self->{type} = "Detailed";
            }
         }
      }
      elsif ( $cmd =~ /monthly/i ) {
         if ( $val eq "Y" ) {
            $self->{monthly} = 1;
            if ( "$type" eq "" ) {
               $self->{type} = "Monthly";
            }
         }
      }
      else {
         $self->{cmd} = $val;
      }
   }

   $self->{beg} =~ s/-/\//g;
   $self->{end} =~ s/-/\//g;
#   print "Have beg = $self->{beg}\n";
#   print "Have end = $self->{end}\n";
   my ($m1, $d1, $y1) = split /\//, $self->{beg};
   $self->{m1} = $m1;
   $self->{d1} = $d1;
   $self->{y1} = $y1;
   
   my ($m2, $d2, $y2) = split /\//, $self->{end};
   $self->{m2} = $m2;
   $self->{d2} = $d2;
   $self->{y2} = $y2;
   

   if ( $self->{out_format} eq "ps" ) {
      $outcmd = "| tbl | groff -me -Tps $ppopts | lpr -P $self->{printer}";
#      $outcmd = ">&STDOUT";
   }
   else {
      $outcmd = ">&STDOUT";
   }
   printf STDERR "outcmd = '$outcmd'\n";
   open OUTF, $outcmd;

   return bless $self, $class;
#   return $self;
}

sub openDatabase
{
   my ($self) = @_;
   # print "beg = $beg, end = $end\n";
   # print "Retrieving Database name from stdin\n";
   ($dbname, $dbhost, $dbport, $dbuser, $dbpasswd) = split /:/, <>;
   chop $passwd;
   # print "dbname = |$dbname|, host = $dbhost, port = |$dbport|, user = |$dbuser|, passwd = |$dbpasswd|\n";

   $dbh = DBI->connect("dbi:Pg:dbname=$dbname host=$dbhost port=$dbport", $dbuser, $dbpasswd);
   return($dbh);
}

sub getDateSpan
{
   my ($self) = @_;
   my $m1 = $self->{m1};
   my $d1 = $self->{d1};
   my $y1 = $self->{y1};
   my $m2 = $self->{m2};
   my $d2 = $self->{d2};
   my $y2 = $self->{y2};
   return("$m1/$d1/$y1", "$m2/$d2/$y2");
}

sub getNextMonth
{
   my ($self) = @_;
   if ( $self->{m3} eq undef ) {
      $self->{m3} = $self->{m1};
      $self->{d3} = $self->{d1};
      $self->{y3} = $self->{y1};
   }
   my $m2 = $self->{m2};
   my $d2 = $self->{d2};
   my $y2 = $self->{y2};
   my $m3 = $self->{m3};
   my $d3 = $self->{d3};
   my $y3 = $self->{y3};

   if ( ($m3 == $m2 and $y3 == $y2)) {
      $rv1 = "$m2/1/$y2";
      $rv2 = "$m2/$d2/$y2";
   }
   else {
      $rv1 = "$m3/1/$y3";
      $rv2 = "$m3/$days[$m3-1]/$y3";
   }
   if ( $self->{done} == 0) {
      $self->{m3}++;
   }
   if ( $m3 > $m2 && $y3 == $y2 ) {
#      print STDERR "All done!\n";
      $self->{done} = 1;
   }
   elsif ($m > 12) {
      $self->{m3} = 1;
      $self->{y3}++;
   }
#   print STDERR "Done = $self->{done}\n";
   return($rv1, $rv2, $self->{done});
}

sub print_rpt
{
   my ($self, $out) = @_;
   print OUTF $out;
}

sub fmt_money
{
   my $self = shift;
   my $amt = shift;
   my $neg = 0;

   print STDERR "fmt_money: Enter ($amt)\n";
   $amt =~ s/[\$\(\),]//g;
   $amt = sprintf("%15.2f", $amt);
   if ( $amt < 0.00 ) {
      $neg = 1;
   }
   print STDERR "fmt_money: amt after sprintf |$amt|\n";
   $amt =~ s/\s+//g;
   while ($amt =~ s/([0-9])([0-9]{3})([.,])/$1,$2$3/) {
      print STDERR "fmt_money: amt = $amt\n";
   }
   $amt = "\$" . $amt;
   if ( $neg > 0 ) {
      $amt = "($amt)";
   }
   return($amt);
}

1
